/* 
 SOME BASIC GAME RULES - easier to have this in front while coding the game logic
 A deck of 52 cards
 Only 26 cards are dealt - 13 per player
 Each player plays one card, the one who has a higher card winds the trick  - this is one round
 the game has 13 rounds
 so whoever makes 7 tricks first has more tricks and thus has won the game
 the player who plays his card second can only play a card of the same suit unless they don't have that suit
 the player who makes the trick plays first in the next round
 */
 
//========================================================================================================================================
// declaring/initializing global variables

//for the font
PFont font; 

int screenMode = 0;
boolean start = false; //to keep track if the game has started or not

//home and mute button
float homeW;
boolean muteToggle = false; //unmute by default
boolean muted; //to change the value of mute toggle

//loading all images
PImage bg;
PImage[] cImgs; //card images
PImage table;
//PImage[] border;
PImage logo;
PImage home;
PImage[] mute;

//Deck deck;
Card[] cards;
int[] p1Hand; //keeps track of the cards in hand for player one (by id)
int[] p2Hand; //keeps track of the cards in hand for player two (by id)
//Card[] back; - can have this seperately

//game screen positions
float cardW, deckW, tCardH, tCardW;
float ratio;
float tableH, tableW; //dimension of the playing table
float marginX, marginY; // to keep track of outside table margins
float p1X, p2X; //to keep track of position where players play their cards
float t1Y, t2Y; //to keep track of position where tricks are kept
float deckPosX, deckPosY; //center coordinates of tthe deck

//variables needed while dealing the cards
boolean dealt = true;
int dealCardNum = 0; //to keep track of which card is being dealt;
float dealIncNum = 4; //how much distance to cover for each dealt card
int[] ctr; //to keep track how much distance my dealt card has travelled
//ctr = new int[26];

//variables needed while playing
int round = 1; //game consists of total 13 rounds (each player has 13 cards)
int turn = 1; //to keep track which player's turn it is
int firstTurn = 1; //to keep track of which player has to start round - no restriction on playing the card
int card1Played; //to keep track of which card was played second
int card2Played; //to keep track of which card was played first - to match the suit
int cardPlayed; //gets index of which card in hand is played
int p1trick = 0; //to keep track of the number of tricks made by player 1
int p2trick = 0; //to keep track of the number of tricks made by player 2

//variables needed to collect tricks
int prevCard1 = 100;
int prevCard2 = 100;
int won = 0; // to keep track of who won the round;
int counter = 0;
float finalY, finalX;
float incX1, incY, incX2;
float trickIncNum = 10;
boolean delay = false;

//for delay
int m, m2;
boolean pause = true;

//adding sound effects
import processing.sound.*;
SoundFile sounds[];
boolean cheer = false; //for cheering sound in the end


//========================================================================================================================================
//SETUP 

void setup()
{
  //canvas size
  size(1200, 800);
  //fullScreen();
  background(255);
  //frameRate(180);
  //loading fonts
  font = loadFont("Apple-Chancery-38.vlw");

  //setting the various positions of game screen
  tableW = width*0.6;
  tableH = height*0.55;
  marginX = (width-tableW)/2;
  marginY = (height-tableH)/2;
  cardW = (tableW+marginX)/13; //for cards in hand
  //ratio = 1056/691; //to determine height of cards depending on cardW
  deckW = marginX*0.6; //for cards in the deck
  tCardH = marginX*0.6; //for the tricks made
  tCardW = tCardH/1056*691;
  p1X = width/2 - deckW;
  p2X = width/2 + deckW;
  t2Y = marginY + 1.2*tCardH/2;
  t1Y = height - marginY - 1.2*tCardH/2;
  deckPosX = marginX/2;
  deckPosY = height/2;

  rectMode(CENTER);
  imageMode(CENTER);
  textAlign(CENTER, CENTER);
  textSize(18);

  //loading different images
  bg = loadImage("bg1.jpg");
  home = loadImage("home2.png");
  homeW = 35;
  table = loadImage("table.jpg");
  logo = loadImage("logo4.png");
  mute = new PImage[2];
  mute[0] = loadImage("unmuted2.png");
  mute[1] = loadImage("muted.png");

  //loading card images
  cImgs = new PImage[53]; // 1 extra for back side
  // to store card image names
  String[] filenames = {"back.png", "AC.png", "2C.png", "3C.png", "4C.png", "5C.png", "6C.png", 
    "7C.png", "8C.png", "9C.png", "10C.png", "JC.png", "QC.png", "KC.png", 
    "AD.png", "2D.png", "3D.png", "4D.png", "5D.png", "6D.png", "7D.png", 
    "8D.png", "9D.png", "10D.png", "JD.png", "QD.png", "KD.png", 
    "AH.png", "2H.png", "3H.png", "4H.png", "5H.png", "6H.png", "7H.png", 
    "8H.png", "9H.png", "10H.png", "JH.png", "QH.png", "KH.png", 
    "AS.png", "2S.png", "3S.png", "4S.png", "5S.png", "6S.png", "7S.png", 
    "8S.png", "9S.png", "10S.png", "JS.png", "QS.png", "KS.png" };
  for (int i=0; i<=52; i++)
    cImgs[i] = loadImage(filenames[i]);

  //creating my card objects
  cards = new Card[53]; //52 open cards + 1 closed card
  cards[0] = new Card(0, 0, "back", cImgs[0]);
  //for clubs - cards 1-13
  for (int i=1; i<=13; i++)
    cards[i] = new Card(i, i, "clubs", cImgs[i]);
  //for diamonds - cards 14-26
  for (int i=14; i<=26; i++)
    cards[i] = new Card(i, i-13, "diamonds", cImgs[i]);
  //for hearts - cards 27-39
  for (int i=27; i<=39; i++)
    cards[i] = new Card(i, i-26, "hearts", cImgs[i]);
  //for spades - cards 40-52
  for (int i=40; i<=52; i++)
    cards[i] = new Card(i, i-39, "spades", cImgs[i]);

  //player hands
  p1Hand = new int[13];
  p2Hand = new int[13];

  //deal
  ctr = new int[26];
  for (int i=0; i<26; i++)
    ctr[i]=0;
  //deal();

  //loading sounds
  sounds = new SoundFile[7];
  sounds[0] = new SoundFile(this, "deal cards single 1.mp3");
  sounds[1] = new SoundFile(this, "cards flip 1.mp3");
  sounds[2] = new SoundFile(this, "pick trick 5.mp3");
  sounds[3] = new SoundFile(this, "next round.mp3");
  sounds[4] = new SoundFile(this, "game over.mp3");
  sounds[5] = new SoundFile(this, "beep1.mp3");
  sounds[6] = new SoundFile(this, "bgM.mp3");
  //sounds[0].amp(0.3);
  //sounds[1].amp(0.25);
  //sounds[3].amp(0.3);
  //sounds[5].amp(0.7);
  //sounds[6].amp(0.6);
  unmute();
  sounds[6].play(); //bg music keeps playing in the background
  sounds[6].loop();
}


//========================================================================================================================================
//DRAW LOOP

void draw()
{
  //background
  image(bg, width/2, height/2, width, height);

  //mute/unmute button
  if (muteToggle == false) //i.e. unmuted
  {
    image(mute[0], width-homeW, homeW, homeW, homeW*1.1);

    if (mouseX>width-homeW-homeW/2 && mouseX<width-homeW+homeW/2 && mouseY>homeW-homeW*1.1/2 && mouseY<homeW+homeW*1.1/2)
    {
      //tint on hover
      tint(0, 0, 0, 70); //change color on hover
      image(mute[0], width-homeW, homeW, homeW, homeW*1.1);
      noTint();
    }
  } else //display thhe muted pic
  {
    image(mute[1], width-homeW, homeW, homeW, homeW*1.1);  

    if (mouseX>width-homeW-homeW/2 && mouseX<width-homeW+homeW/2 && mouseY>homeW-homeW*1.1/2 && mouseY<homeW+homeW*1.1/2)
    {
      //tint on hover
      tint(0, 0, 0, 70); //change color on hover
      image(mute[1], width-homeW, homeW, homeW, homeW*1.1);
      noTint();
    }
  }

  //to display appropriate screen
  switch (screenMode)
  {
  case 0 :  
    if (muteToggle == false)
      sounds[6].amp(0.5);
    startScreen();    
    break;
  case 1 : 
    if (muteToggle == false)
      sounds[6].amp(0.05);
    gameScreen(); 
    break;
  case 3 : 
    popUp(); //to ask whether restart or continue the game in progress
    break;
  case 2 : 
    instScreen(); 
    break; //instruction screen
  }
}

//========================================================================================================================================

void mute()
{
  for (int i=0; i<sounds.length; i++)
    sounds[i].amp(0);
  muteToggle = true;
}

void unmute()
{
  sounds[0].amp(0.3);
  sounds[1].amp(0.25);
  sounds[2].amp(1);
  sounds[3].amp(0.3);
  sounds[4].amp(0.5);
  sounds[5].amp(0.7);
  sounds[6].amp(0.5);
  muteToggle = false;
}

//========================================================================================================================================

void mouseClicked()
{
  //for all screens
  //change icon of mute/unmute button upon click
  if (mouseX>width-homeW-homeW/2 && mouseX<width-homeW+homeW/2 && mouseY>homeW-homeW*1.1/2 && mouseY<homeW+homeW*1.1/2)
  {
    if (muteToggle == false) //i.e. unmuted
      mute();
    else //i.e. unmuted
    unmute();
  }

  //clicks on the pop-up screen
  if (screenMode == 3)
  {
    //for button 1 = continue
    if (mouseX<width/2-width*0.1+marginX*0.8/2 && mouseX>width/2-width*0.1-marginX*0.8/2
      && mouseY<height/2+height*0.08+marginY*0.28/2 && mouseY>height/2+height*0.08-marginY*0.28/2)
    {
      screenMode = 1;
      m2 = millis() - 500; //dont need delay when continuing
    }
    //for button 2 = restart
    if (mouseX<width/2+width*0.1+marginX*0.8/2 && mouseX>width/2+width*0.1-marginX*0.8/2
      && mouseY<height/2+height*0.08+marginY*0.28/2 && mouseY>height/2+height*0.08-marginY*0.28/2)
    {
      start = false;
      screenMode = 1;
      m2 = millis();
    }
  }

  //clicks on the start screen
  if (screenMode == 0)
  {
    //change to game screen
    if (mouseX>width/2-width*0.3/2 && mouseX<width/2+width*0.3/2 && mouseY>height*0.6-marginY*0.4/2 && mouseY<height*0.6+marginY*0.4/2)
    {
      if (start) //game has already started - provide with o0ption to resume or restart
        screenMode = 3;
      else
      {
        screenMode = 1;
        m2 = millis();
      }
    }
    if (mouseX>width/2-width*0.3/2 && mouseX<width/2+width*0.3/2 && mouseY>height*0.75-marginY*0.4/2 && mouseY<height*0.75+marginY*0.4/2)
      screenMode = 2;
  }

  //click on home button on instructions screen
  if (screenMode == 2)
  {
    if (mouseX>homeW*0.9-homeW/2 && mouseX<homeW*0.9+homeW/2 && mouseY>homeW*0.9-homeW/2 && mouseY<homeW*0.9+homeW/2)
      screenMode = 0;
  }

  //click on home button on game screen
  if (screenMode == 1)
    if (mouseX>homeW*0.9-homeW/2 && mouseX<homeW*0.9+homeW/2 && mouseY>homeW*0.9-homeW/2 && mouseY<homeW*0.9+homeW/2)
    {
      if (round>13) //if game over, then when you go back to home, should ot ask to resume game
        start = false;
      sounds[4].stop(); //stop the cheering sound if still playing but user goes back to home
      screenMode = 0;
    }

  //for the play again button on the game screen when game finishes
  if (screenMode == 1)
    if (round>13 && pause)
      if (mouseX<width/2+marginX*0.8/2 && mouseX>width/2-marginX*0.8/2
        && mouseY<height-marginY/2+marginY*0.3/2 && mouseY>height-marginY/2-marginY*0.3/2)
      {
        start = false;
        sounds[4].stop(); //stop playing the cheering sound even if it hasnt finished -- do same when click on home button
      }
}
